@@ -116,15 +116,13 @@ func runMount(opts MountOptions, gopts GlobalOptions, args []string) error {
 	mountpoint := args[0]
 
 	if _, err := resticfs.Stat(mountpoint); os.IsNotExist(errors.Cause(err)) {
-		Verbosef("Mountpoint %s doesn't exist, creating it\n", mountpoint)
-		err = resticfs.Mkdir(mountpoint, os.ModeDir|0700)
-		if err != nil {
-			return err
-		}
+		Verbosef("Mountpoint %s doesn't exist\n", mountpoint)
+		return err
 	}
 	mountOptions := []systemFuse.MountOption{
 		systemFuse.ReadOnly(),
 		systemFuse.FSName("restic"),
+		systemFuse.MaxReadahead(128 * 1024),
 	}
 
 	if opts.AllowOther {
