@@ -13,6 +13,8 @@ import (
 	"github.com/restic/restic/internal/backend"
 	"github.com/restic/restic/internal/debug"
 	"github.com/restic/restic/internal/fs"
+
+	"github.com/cenkalti/backoff/v4"
 )
 
 // Local is a backend in a local directory.
@@ -62,7 +64,7 @@ func Create(ctx context.Context, cfg Config) (*Local, error) {
 	for _, d := range be.Paths() {
 		err := fs.MkdirAll(d, backend.Modes.Dir)
 		if err != nil {
-			return nil, errors.Wrap(err, "MkdirAll")
+			return nil, errors.WithStack(err)
 		}
 	}
 
@@ -76,20 +78,27 @@ func (b *Local) Location() string {
 
 // IsNotExist returns true if the error is caused by a non existing file.
 func (b *Local) IsNotExist(err error) bool {
-	return os.IsNotExist(errors.Cause(err))
+	return errors.Is(err, os.ErrNotExist)
 }
 
 // Save stores data in the backend at the handle.
-func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReader) error {
+func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReader) (err error) {
 	debug.Log("Save %v", h)
 	if err := h.Valid(); err != nil {
-		return err
+		return backoff.Permanent(err)
 	}
 
 	filename := b.Filename(h)
 
+	defer func() {
+		// Mark non-retriable errors as such
+		if errors.Is(err, syscall.ENOSPC) || os.IsPermission(err) {
+			err = backoff.Permanent(err)
+		}
+	}()
+
 	// create new file
-	f, err := fs.OpenFile(filename, os.O_CREATE|os.O_EXCL|os.O_WRONLY, backend.Modes.File)
+	f, err := openFile(filename, os.O_CREATE|os.O_EXCL|os.O_WRONLY, backend.Modes.File)
 
 	if b.IsNotExist(err) {
 		debug.Log("error %v: creating dir", err)
@@ -100,19 +109,24 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 			debug.Log("error creating dir %v: %v", filepath.Dir(filename), mkdirErr)
 		} else {
 			// try again
-			f, err = fs.OpenFile(filename, os.O_CREATE|os.O_EXCL|os.O_WRONLY, backend.Modes.File)
+			f, err = openFile(filename, os.O_CREATE|os.O_EXCL|os.O_WRONLY, backend.Modes.File)
 		}
 	}
 
 	if err != nil {
-		return errors.Wrap(err, "OpenFile")
+		return errors.WithStack(err)
 	}
 
 	// save data, then sync
-	_, err = io.Copy(f, rd)
+	wbytes, err := io.Copy(f, rd)
 	if err != nil {
 		_ = f.Close()
-		return errors.Wrap(err, "Write")
+		return errors.WithStack(err)
+	}
+	// sanity check
+	if wbytes != rd.Length() {
+		_ = f.Close()
+		return errors.Errorf("wrote %d bytes instead of the expected %d bytes", wbytes, rd.Length())
 	}
 
 	if err = f.Sync(); err != nil {
@@ -121,13 +135,13 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 		// ignore error if filesystem does not support the sync operation
 		if !isNotSupported {
 			_ = f.Close()
-			return errors.Wrap(err, "Sync")
+			return errors.WithStack(err)
 		}
 	}
 
 	err = f.Close()
 	if err != nil {
-		return errors.Wrap(err, "Close")
+		return errors.WithStack(err)
 	}
 
 	// try to mark file as read-only to avoid accidential modifications
@@ -135,12 +149,14 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 	// e.g. exfat and network file systems with certain mount options
 	err = setFileReadonly(filename, backend.Modes.File)
 	if err != nil && !os.IsPermission(err) {
-		return errors.Wrap(err, "Chmod")
+		return errors.WithStack(err)
 	}
 
 	return nil
 }
 
+var openFile = fs.OpenFile // Overridden by test.
+
 // Load runs fn with a reader that yields the contents of the file at h at the
 // given offset.
 func (b *Local) Load(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
@@ -150,7 +166,7 @@ func (b *Local) Load(ctx context.Context, h restic.Handle, length int, offset in
 func (b *Local) openReader(ctx context.Context, h restic.Handle, length int, offset int64) (io.ReadCloser, error) {
 	debug.Log("Load %v, length %v, offset %v", h, length, offset)
 	if err := h.Valid(); err != nil {
-		return nil, err
+		return nil, backoff.Permanent(err)
 	}
 
 	if offset < 0 {
@@ -181,12 +197,12 @@ func (b *Local) openReader(ctx context.Context, h restic.Handle, length int, off
 func (b *Local) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, error) {
 	debug.Log("Stat %v", h)
 	if err := h.Valid(); err != nil {
-		return restic.FileInfo{}, err
+		return restic.FileInfo{}, backoff.Permanent(err)
 	}
 
 	fi, err := fs.Stat(b.Filename(h))
 	if err != nil {
-		return restic.FileInfo{}, errors.Wrap(err, "Stat")
+		return restic.FileInfo{}, errors.WithStack(err)
 	}
 
 	return restic.FileInfo{Size: fi.Size(), Name: h.Name}, nil
@@ -197,10 +213,10 @@ func (b *Local) Test(ctx context.Context, h restic.Handle) (bool, error) {
 	debug.Log("Test %v", h)
 	_, err := fs.Stat(b.Filename(h))
 	if err != nil {
-		if os.IsNotExist(errors.Cause(err)) {
+		if b.IsNotExist(err) {
 			return false, nil
 		}
-		return false, errors.Wrap(err, "Stat")
+		return false, errors.WithStack(err)
 	}
 
 	return true, nil
@@ -214,58 +230,23 @@ func (b *Local) Remove(ctx context.Context, h restic.Handle) error {
 	// reset read-only flag
 	err := fs.Chmod(fn, 0666)
 	if err != nil && !os.IsPermission(err) {
-		return errors.Wrap(err, "Chmod")
+		return errors.WithStack(err)
 	}
 
 	return fs.Remove(fn)
 }
 
-func isFile(fi os.FileInfo) bool {
-	return fi.Mode()&(os.ModeType|os.ModeCharDevice) == 0
-}
-
 // List runs fn for each file in the backend which has the type t. When an
 // error occurs (or fn returns an error), List stops and returns it.
-func (b *Local) List(ctx context.Context, t restic.FileType, fn func(restic.FileInfo) error) error {
+func (b *Local) List(ctx context.Context, t restic.FileType, fn func(restic.FileInfo) error) (err error) {
 	debug.Log("List %v", t)
 
 	basedir, subdirs := b.Basedir(t)
-	err := fs.Walk(basedir, func(path string, fi os.FileInfo, err error) error {
-		debug.Log("walk on %v\n", path)
-		if err != nil {
-			return err
-		}
-
-		if path == basedir {
-			return nil
-		}
-
-		if !isFile(fi) {
-			return nil
-		}
-
-		if fi.IsDir() && !subdirs {
-			return filepath.SkipDir
-		}
-
-		debug.Log("send %v\n", filepath.Base(path))
-
-		rfi := restic.FileInfo{
-			Name: filepath.Base(path),
-			Size: fi.Size(),
-		}
-
-		if ctx.Err() != nil {
-			return ctx.Err()
-		}
-
-		err = fn(rfi)
-		if err != nil {
-			return err
-		}
-
-		return ctx.Err()
-	})
+	if subdirs {
+		err = visitDirs(ctx, basedir, fn)
+	} else {
+		err = visitFiles(ctx, basedir, fn, false)
+	}
 
 	if b.IsNotExist(err) {
 		debug.Log("ignoring non-existing directory")
@@ -275,6 +256,80 @@ func (b *Local) List(ctx context.Context, t restic.FileType, fn func(restic.File
 	return err
 }
 
+// The following two functions are like filepath.Walk, but visit only one or
+// two levels of directory structure (including dir itself as the first level).
+// Also, visitDirs assumes it sees a directory full of directories, while
+// visitFiles wants a directory full or regular files.
+func visitDirs(ctx context.Context, dir string, fn func(restic.FileInfo) error) error {
+	d, err := fs.Open(dir)
+	if err != nil {
+		return err
+	}
+
+	sub, err := d.Readdirnames(-1)
+	if err != nil {
+		// ignore subsequent errors
+		_ = d.Close()
+		return err
+	}
+
+	err = d.Close()
+	if err != nil {
+		return err
+	}
+
+	for _, f := range sub {
+		err = visitFiles(ctx, filepath.Join(dir, f), fn, true)
+		if err != nil {
+			return err
+		}
+	}
+	return ctx.Err()
+}
+
+func visitFiles(ctx context.Context, dir string, fn func(restic.FileInfo) error, ignoreNotADirectory bool) error {
+	d, err := fs.Open(dir)
+	if err != nil {
+		return err
+	}
+
+	if ignoreNotADirectory {
+		fi, err := d.Stat()
+		if err != nil || !fi.IsDir() {
+			return err
+		}
+	}
+
+	sub, err := d.Readdir(-1)
+	if err != nil {
+		// ignore subsequent errors
+		_ = d.Close()
+		return err
+	}
+
+	err = d.Close()
+	if err != nil {
+		return err
+	}
+
+	for _, fi := range sub {
+		select {
+		case <-ctx.Done():
+			return ctx.Err()
+		default:
+		}
+
+		err := fn(restic.FileInfo{
+			Name: fi.Name(),
+			Size: fi.Size(),
+		})
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // Delete removes the repository and all files.
 func (b *Local) Delete(ctx context.Context) error {
 	debug.Log("Delete()")
