@@ -1,7 +1,6 @@
 package pack
 
 import (
-	"bytes"
 	"encoding/binary"
 	"fmt"
 	"io"
@@ -36,7 +35,7 @@ func (p *Packer) Add(t restic.BlobType, id restic.ID, data []byte) (int, error)
 	p.m.Lock()
 	defer p.m.Unlock()
 
-	c := restic.Blob{Type: t, ID: id}
+	c := restic.Blob{BlobHandle: restic.BlobHandle{Type: t, ID: id}}
 
 	n, err := p.wr.Write(data)
 	c.Length = uint(n)
@@ -47,9 +46,10 @@ func (p *Packer) Add(t restic.BlobType, id restic.ID, data []byte) (int, error)
 	return n, errors.Wrap(err, "Write")
 }
 
-var entrySize = uint(binary.Size(restic.BlobType(0)) + binary.Size(uint32(0)) + len(restic.ID{}))
+var EntrySize = uint(binary.Size(restic.BlobType(0)) + headerLengthSize + len(restic.ID{}))
 
-// headerEntry is used with encoding/binary to read and write header entries
+// headerEntry describes the format of header entries. It serves only as
+// documentation.
 type headerEntry struct {
 	Type   uint8
 	Length uint32
@@ -64,16 +64,15 @@ func (p *Packer) Finalize() (uint, error) {
 
 	bytesWritten := p.bytes
 
-	hdrBuf := bytes.NewBuffer(nil)
-	bytesHeader, err := p.writeHeader(hdrBuf)
+	header, err := p.makeHeader()
 	if err != nil {
 		return 0, err
 	}
 
-	encryptedHeader := make([]byte, 0, hdrBuf.Len()+p.k.Overhead()+p.k.NonceSize())
+	encryptedHeader := make([]byte, 0, len(header)+p.k.Overhead()+p.k.NonceSize())
 	nonce := crypto.NewRandomNonce()
 	encryptedHeader = append(encryptedHeader, nonce...)
-	encryptedHeader = p.k.Seal(encryptedHeader, nonce, hdrBuf.Bytes(), nil)
+	encryptedHeader = p.k.Seal(encryptedHeader, nonce, header, nil)
 
 	// append the header
 	n, err := p.wr.Write(encryptedHeader)
@@ -81,7 +80,7 @@ func (p *Packer) Finalize() (uint, error) {
 		return 0, errors.Wrap(err, "Write")
 	}
 
-	hdrBytes := restic.CiphertextLength(int(bytesHeader))
+	hdrBytes := restic.CiphertextLength(len(header))
 	if n != hdrBytes {
 		return 0, errors.New("wrong number of bytes written")
 	}
@@ -89,7 +88,7 @@ func (p *Packer) Finalize() (uint, error) {
 	bytesWritten += uint(hdrBytes)
 
 	// write length
-	err = binary.Write(p.wr, binary.LittleEndian, uint32(restic.CiphertextLength(len(p.blobs)*int(entrySize))))
+	err = binary.Write(p.wr, binary.LittleEndian, uint32(restic.CiphertextLength(len(p.blobs)*int(EntrySize))))
 	if err != nil {
 		return 0, errors.Wrap(err, "binary.Write")
 	}
@@ -99,32 +98,27 @@ func (p *Packer) Finalize() (uint, error) {
 	return bytesWritten, nil
 }
 
-// writeHeader constructs and writes the header to wr.
-func (p *Packer) writeHeader(wr io.Writer) (bytesWritten uint, err error) {
-	for _, b := range p.blobs {
-		entry := headerEntry{
-			Length: uint32(b.Length),
-			ID:     b.ID,
-		}
+// makeHeader constructs the header for p.
+func (p *Packer) makeHeader() ([]byte, error) {
+	buf := make([]byte, 0, len(p.blobs)*int(EntrySize))
 
+	for _, b := range p.blobs {
 		switch b.Type {
 		case restic.DataBlob:
-			entry.Type = 0
+			buf = append(buf, 0)
 		case restic.TreeBlob:
-			entry.Type = 1
+			buf = append(buf, 1)
 		default:
-			return 0, errors.Errorf("invalid blob type %v", b.Type)
+			return nil, errors.Errorf("invalid blob type %v", b.Type)
 		}
 
-		err := binary.Write(wr, binary.LittleEndian, entry)
-		if err != nil {
-			return bytesWritten, errors.Wrap(err, "binary.Write")
-		}
-
-		bytesWritten += entrySize
+		var lenLE [4]byte
+		binary.LittleEndian.PutUint32(lenLE[:], uint32(b.Length))
+		buf = append(buf, lenLE[:]...)
+		buf = append(buf, b.ID[:]...)
 	}
 
-	return
+	return buf, nil
 }
 
 // Size returns the number of bytes written so far.
@@ -151,23 +145,21 @@ func (p *Packer) Blobs() []restic.Blob {
 	return p.blobs
 }
 
-// Writer return the underlying writer.
-func (p *Packer) Writer() io.Writer {
-	return p.wr
-}
-
 func (p *Packer) String() string {
 	return fmt.Sprintf("<Packer %d blobs, %d bytes>", len(p.blobs), p.bytes)
 }
 
 var (
-	// size of the header-length field at the end of the file
-	headerLengthSize = binary.Size(uint32(0))
 	// we require at least one entry in the header, and one blob for a pack file
-	minFileSize = entrySize + crypto.Extension + uint(headerLengthSize)
+	minFileSize = EntrySize + crypto.Extension + uint(headerLengthSize)
 )
 
 const (
+	// size of the header-length field at the end of the file; it is a uint32
+	headerLengthSize = 4
+	// HeaderSize is the header's constant overhead (independent of #entries)
+	HeaderSize = headerLengthSize + crypto.Extension
+
 	maxHeaderSize = 16 * 1024 * 1024
 	// number of header enries to download as part of header-length request
 	eagerEntries = 15
@@ -179,7 +171,7 @@ const (
 // the appropriate size.
 func readRecords(rd io.ReaderAt, size int64, max int) ([]byte, int, error) {
 	var bufsize int
-	bufsize += max * int(entrySize)
+	bufsize += max * int(EntrySize)
 	bufsize += crypto.Extension
 	bufsize += headerLengthSize
 
@@ -203,7 +195,7 @@ func readRecords(rd io.ReaderAt, size int64, max int) ([]byte, int, error) {
 		err = InvalidFileError{Message: "header length is zero"}
 	case hlen < crypto.Extension:
 		err = InvalidFileError{Message: "header length is too small"}
-	case (hlen-crypto.Extension)%uint32(entrySize) != 0:
+	case (hlen-crypto.Extension)%uint32(EntrySize) != 0:
 		err = InvalidFileError{Message: "header length is invalid"}
 	case int64(hlen) > size-int64(headerLengthSize):
 		err = InvalidFileError{Message: "header is larger than file"}
@@ -214,7 +206,7 @@ func readRecords(rd io.ReaderAt, size int64, max int) ([]byte, int, error) {
 		return nil, 0, errors.Wrap(err, "readHeader")
 	}
 
-	total := (int(hlen) - crypto.Extension) / int(entrySize)
+	total := (int(hlen) - crypto.Extension) / int(EntrySize)
 	if total < max {
 		// truncate to the beginning of the pack header
 		b = b[len(b)-int(hlen):]
@@ -260,58 +252,67 @@ func (e InvalidFileError) Error() string {
 	return e.Message
 }
 
-// List returns the list of entries found in a pack file.
-func List(k *crypto.Key, rd io.ReaderAt, size int64) (entries []restic.Blob, err error) {
+// List returns the list of entries found in a pack file and the length of the
+// header (including header size and crypto overhead)
+func List(k *crypto.Key, rd io.ReaderAt, size int64) (entries []restic.Blob, hdrSize uint32, err error) {
 	buf, err := readHeader(rd, size)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	if len(buf) < k.NonceSize()+k.Overhead() {
-		return nil, errors.New("invalid header, too small")
+		return nil, 0, errors.New("invalid header, too small")
 	}
 
+	hdrSize = headerLengthSize + uint32(len(buf))
+
 	nonce, buf := buf[:k.NonceSize()], buf[k.NonceSize():]
 	buf, err = k.Open(buf[:0], nonce, buf, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	hdrRd := bytes.NewReader(buf)
-
-	entries = make([]restic.Blob, 0, uint(len(buf))/entrySize)
+	entries = make([]restic.Blob, 0, uint(len(buf))/EntrySize)
 
 	pos := uint(0)
-	for {
-		e := headerEntry{}
-		err = binary.Read(hdrRd, binary.LittleEndian, &e)
-		if errors.Cause(err) == io.EOF {
-			break
-		}
-
+	for len(buf) > 0 {
+		entry, err := parseHeaderEntry(buf)
 		if err != nil {
-			return nil, errors.Wrap(err, "binary.Read")
-		}
-
-		entry := restic.Blob{
-			Length: uint(e.Length),
-			ID:     e.ID,
-			Offset: pos,
-		}
-
-		switch e.Type {
-		case 0:
-			entry.Type = restic.DataBlob
-		case 1:
-			entry.Type = restic.TreeBlob
-		default:
-			return nil, errors.Errorf("invalid type %d", e.Type)
+			return nil, 0, err
 		}
+		entry.Offset = pos
 
 		entries = append(entries, entry)
-
-		pos += uint(e.Length)
+		pos += entry.Length
+		buf = buf[EntrySize:]
 	}
 
-	return entries, nil
+	return entries, hdrSize, nil
+}
+
+// PackedSizeOfBlob returns the size a blob actually uses when saved in a pack
+func PackedSizeOfBlob(blobLength uint) uint {
+	return blobLength + EntrySize
+}
+
+func parseHeaderEntry(p []byte) (b restic.Blob, err error) {
+	if uint(len(p)) < EntrySize {
+		err = errors.Errorf("parseHeaderEntry: buffer of size %d too short", len(p))
+		return b, err
+	}
+	p = p[:EntrySize]
+
+	switch p[0] {
+	case 0:
+		b.Type = restic.DataBlob
+	case 1:
+		b.Type = restic.TreeBlob
+	default:
+		return b, errors.Errorf("invalid type %d", p[0])
+	}
+
+	b.Length = uint(binary.LittleEndian.Uint32(p[1:5]))
+	copy(b.ID[:], p[5:])
+
+	return b, nil
 }
