@@ -32,7 +32,10 @@ type Repository interface {
 	//
 	// The function fn is called in the same Goroutine List() was called from.
 	List(ctx context.Context, t FileType, fn func(ID, int64) error) error
-	ListPack(context.Context, ID, int64) ([]Blob, int64, error)
+
+	// ListPack returns the list of blobs saved in the pack id and the length of
+	// the the pack header.
+	ListPack(context.Context, ID, int64) ([]Blob, uint32, error)
 
 	Flush(context.Context) error
 
@@ -59,9 +62,10 @@ type Lister interface {
 
 // MasterIndex keeps track of the blobs are stored within files.
 type MasterIndex interface {
-	Has(ID, BlobType) bool
-	Lookup(ID, BlobType) []PackedBlob
+	Has(BlobHandle) bool
+	Lookup(BlobHandle) []PackedBlob
 	Count(BlobType) uint
+	PackSize(ctx context.Context, onlyHdr bool) map[ID]int64
 
 	// Each returns a channel that yields all blobs known to the index. When
 	// the context is cancelled, the background goroutine terminates. This
