@@ -3,11 +3,13 @@ package restorer
 import (
 	"bytes"
 	"context"
+	"fmt"
 	"io"
 	"io/ioutil"
 	"testing"
 
 	"github.com/restic/restic/internal/crypto"
+	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/restic"
 	rtest "github.com/restic/restic/internal/test"
 )
@@ -39,19 +41,11 @@ type TestRepo struct {
 	loader func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error
 }
 
-func (i *TestRepo) Lookup(blobID restic.ID, _ restic.BlobType) []restic.PackedBlob {
-	packs := i.blobs[blobID]
+func (i *TestRepo) Lookup(bh restic.BlobHandle) []restic.PackedBlob {
+	packs := i.blobs[bh.ID]
 	return packs
 }
 
-func (i *TestRepo) packName(pack *packInfo) string {
-	return i.packsIDToName[pack.id]
-}
-
-func (i *TestRepo) packID(name string) restic.ID {
-	return i.packsNameToID[name]
-}
-
 func (i *TestRepo) fileContent(file *fileInfo) string {
 	return i.filesPathToContent[file.location]
 }
@@ -92,8 +86,10 @@ func newTestRepo(content []TestFile) *TestRepo {
 			if _, found := pack.blobs[blobID]; !found {
 				blobData := seal([]byte(blob.data))
 				pack.blobs[blobID] = restic.Blob{
-					Type:   restic.DataBlob,
-					ID:     blobID,
+					BlobHandle: restic.BlobHandle{
+						Type: restic.DataBlob,
+						ID:   blobID,
+					},
 					Length: uint(len(blobData)),
 					Offset: uint(len(pack.data)),
 				}
@@ -150,16 +146,29 @@ func newTestRepo(content []TestFile) *TestRepo {
 	return repo
 }
 
-func restoreAndVerify(t *testing.T, tempdir string, content []TestFile) {
+func restoreAndVerify(t *testing.T, tempdir string, content []TestFile, files map[string]bool) {
 	repo := newTestRepo(content)
 
 	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup)
-	r.files = repo.files
+
+	if files == nil {
+		r.files = repo.files
+	} else {
+		for _, file := range repo.files {
+			if files[file.location] {
+				r.files = append(r.files, file)
+			}
+		}
+	}
 
 	err := r.restoreFiles(context.TODO())
 	rtest.OK(t, err)
 
-	for _, file := range repo.files {
+	verifyRestore(t, r, repo)
+}
+
+func verifyRestore(t *testing.T, r *fileRestorer, repo *TestRepo) {
+	for _, file := range r.files {
 		target := r.targetPath(file.location)
 		data, err := ioutil.ReadFile(target)
 		if err != nil {
@@ -201,5 +210,108 @@ func TestFileRestorerBasic(t *testing.T) {
 				{"data3-1", "pack3-1"},
 			},
 		},
-	})
+	}, nil)
+}
+
+func TestFileRestorerPackSkip(t *testing.T) {
+	tempdir, cleanup := rtest.TempDir(t)
+	defer cleanup()
+
+	files := make(map[string]bool)
+	files["file2"] = true
+
+	restoreAndVerify(t, tempdir, []TestFile{
+		{
+			name: "file1",
+			blobs: []TestBlob{
+				{"data1-1", "pack1"},
+				{"data1-2", "pack1"},
+				{"data1-3", "pack1"},
+				{"data1-4", "pack1"},
+				{"data1-5", "pack1"},
+				{"data1-6", "pack1"},
+			},
+		},
+		{
+			name: "file2",
+			blobs: []TestBlob{
+				// file is contained in pack1 but need pack parts to be skipped
+				{"data1-2", "pack1"},
+				{"data1-4", "pack1"},
+				{"data1-6", "pack1"},
+			},
+		},
+	}, files)
+}
+
+func TestErrorRestoreFiles(t *testing.T) {
+	tempdir, cleanup := rtest.TempDir(t)
+	defer cleanup()
+	content := []TestFile{
+		{
+			name: "file1",
+			blobs: []TestBlob{
+				{"data1-1", "pack1-1"},
+			},
+		}}
+
+	repo := newTestRepo(content)
+
+	loadError := errors.New("load error")
+	// loader always returns an error
+	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
+		return loadError
+	}
+
+	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup)
+	r.files = repo.files
+
+	err := r.restoreFiles(context.TODO())
+	rtest.Equals(t, loadError, err)
+}
+
+func TestDownloadError(t *testing.T) {
+	for i := 0; i < 100; i += 10 {
+		testPartialDownloadError(t, i)
+	}
+}
+
+func testPartialDownloadError(t *testing.T, part int) {
+	tempdir, cleanup := rtest.TempDir(t)
+	defer cleanup()
+	content := []TestFile{
+		{
+			name: "file1",
+			blobs: []TestBlob{
+				{"data1-1", "pack1"},
+				{"data1-2", "pack1"},
+				{"data1-3", "pack1"},
+			},
+		}}
+
+	repo := newTestRepo(content)
+
+	// loader always returns an error
+	loader := repo.loader
+	repo.loader = func(ctx context.Context, h restic.Handle, length int, offset int64, fn func(rd io.Reader) error) error {
+		// only load partial data to execise fault handling in different places
+		err := loader(ctx, h, length*part/100, offset, fn)
+		if err == nil {
+			return nil
+		}
+		fmt.Println("Retry after error", err)
+		return loader(ctx, h, length, offset, fn)
+	}
+
+	r := newFileRestorer(tempdir, repo.loader, repo.key, repo.Lookup)
+	r.files = repo.files
+	r.Error = func(s string, e error) error {
+		// ignore errors as in the `restore` command
+		fmt.Println("error during restore", s, e)
+		return nil
+	}
+
+	err := r.restoreFiles(context.TODO())
+	rtest.OK(t, err)
+	verifyRestore(t, r, repo)
 }
