@@ -97,7 +97,8 @@ func (w *filesWriter) writeToFile(path string, blob []byte, offset int64, create
 	_, err = wr.WriteAt(blob, offset)
 
 	if err != nil {
-		releaseWriter(wr)
+		// ignore subsequent errors
+		_ = releaseWriter(wr)
 		return err
 	}
 
