@@ -4,6 +4,7 @@ import (
 	"os"
 	"path"
 	"strconv"
+	"syscall"
 	"testing"
 
 	"github.com/restic/restic/internal/fs"
@@ -19,9 +20,14 @@ func TestPreallocate(t *testing.T) {
 			flags := os.O_CREATE | os.O_TRUNC | os.O_WRONLY
 			wr, err := os.OpenFile(path.Join(dirpath, "test"), flags, 0600)
 			test.OK(t, err)
-			defer wr.Close()
+			defer func() {
+				test.OK(t, wr.Close())
+			}()
 
 			err = preallocateFile(wr, i)
+			if err == syscall.ENOTSUP {
+				t.SkipNow()
+			}
 			test.OK(t, err)
 
 			fi, err := wr.Stat()
