@@ -74,7 +74,7 @@ func saveDir(t testing.TB, repo restic.Repository, nodes map[string]Node, inode
 			if mode == 0 {
 				mode = 0644
 			}
-			tree.Insert(&restic.Node{
+			err := tree.Insert(&restic.Node{
 				Type:    "file",
 				Mode:    mode,
 				ModTime: node.ModTime,
@@ -86,6 +86,7 @@ func saveDir(t testing.TB, repo restic.Repository, nodes map[string]Node, inode
 				Inode:   fi,
 				Links:   lc,
 			})
+			rtest.OK(t, err)
 		case Dir:
 			id := saveDir(t, repo, node.Nodes, inode)
 
@@ -94,7 +95,7 @@ func saveDir(t testing.TB, repo restic.Repository, nodes map[string]Node, inode
 				mode = 0755
 			}
 
-			tree.Insert(&restic.Node{
+			err := tree.Insert(&restic.Node{
 				Type:    "dir",
 				Mode:    mode,
 				ModTime: node.ModTime,
@@ -103,6 +104,7 @@ func saveDir(t testing.TB, repo restic.Repository, nodes map[string]Node, inode
 				GID:     uint32(os.Getgid()),
 				Subtree: &id,
 			})
+			rtest.OK(t, err)
 		default:
 			t.Fatalf("unknown node type %T", node)
 		}
@@ -141,12 +143,6 @@ func saveSnapshot(t testing.TB, repo restic.Repository, snapshot Snapshot) (*res
 	return sn, id
 }
 
-// toSlash converts the OS specific path dir to a slash-separated path.
-func toSlash(dir string) string {
-	data := strings.Split(dir, string(filepath.Separator))
-	return strings.Join(data, "/")
-}
-
 func TestRestorer(t *testing.T) {
 	var tests = []struct {
 		Snapshot
@@ -354,7 +350,7 @@ func TestRestorer(t *testing.T) {
 
 			errors := make(map[string]map[string]struct{})
 			res.Error = func(location string, err error) error {
-				location = toSlash(location)
+				location = filepath.ToSlash(location)
 				t.Logf("restore returned error for %q: %v", location, err)
 				if errors[location] == nil {
 					errors[location] = make(map[string]struct{})
